@@ -1,10 +1,14 @@
 from snapshottest import TestCase
 
 from rescape_python_helpers.functional.ramda import to_dict_deep, all_pass_dict, flatten_dct, map_keys_deep, \
-    map_with_obj_deep
+    map_with_obj_deep, pick_deep, unflatten_dct, fake_lens_path_view, key_string_to_lens_path, props, fake_lens_path_set
 from . import ramda as R
 
 
+def unflatten_obj(flat_dct):
+    pass
+
+
 class TestRamda(TestCase):
 
     def test_filter_dict(self):
@@ -40,8 +44,20 @@ class TestRamda(TestCase):
 
     def test_omit_deep(self):
         omit_keys = ['foo', 'bar']
-        dct = dict(foo=1, bar=2, car=dict(foo=3, bar=4, tar=5, pepper=[[dict(achoo=1, bar=2), dict(kale=1, foo=2)]]))
-        assert R.omit_deep(omit_keys, dct) == dict(car=dict(tar=5, pepper=[[dict(achoo=1), dict(kale=1)]]))
+        dct = dict(marty={}, foo=1, bar=2,
+                   car=dict(foo=3, bar=4, tar=5, pepper=[[dict(achoo=1, bar=2), dict(kale=1, foo=2)]]))
+        assert R.omit_deep(omit_keys, dct) == dict(marty={}, car=dict(tar=5, pepper=[[dict(achoo=1), dict(kale=1)]]))
+
+    def test_pick_deep(self):
+        pick_out = dict(billy=dict(goat=['takes', 'a', 'bite'], of=dict(tire=True, shoe={}, popsicle=None)))
+        assert pick_deep(
+            pick_out,
+            dict(
+                billy=dict(goat=['takes', 'a', 'kite'],
+                           of=dict(tire=True, shoe=dict(default='left'), fruit=None, popsicle='Sweet')),
+                andthe=dict(cow=['jumps', 'over', 'the', 'coon'])
+            )
+        ) == dict(billy=dict(goat=['takes', 'a', 'kite'], of=dict(tire=True, shoe={}, popsicle='Sweet')))
 
     def test_filter_deep(self):
         # This should pass
@@ -145,8 +161,36 @@ class TestRamda(TestCase):
                 'iss__there__listen__1': 'story'
                 } == result
 
+    def test_unflatten_dct(self):
+        dct = dict(iss=dict(there=dict(anybody=['willing', 'to'], listen=[dict(to='my'), 'story'])))
+        flat_dct = flatten_dct(dct, '.')
+        assert dct == unflatten_dct(flat_dct)
+
+    # TODO Failing
     def test_map_with_obj_deep(self):
-        assert {'a': [{'b': [[[0, ['cCool']], [1, [{'d': ['fCool']}]]]]}]} == map_with_obj_deep(
+        assert {
+                   'a': [
+                       {'b': [
+                           [
+                               [
+                                   0,
+                                   ['cCool']
+                               ],
+                               [
+                                   1,
+                                   [
+                                       {'d':
+                                           [
+                                               'fCool'
+                                           ]
+                                       }
+                                   ]
+                               ]
+                           ]
+                       ]
+                       }
+                   ]
+               } == map_with_obj_deep(
             lambda k, v: [f'{v}Cool' if isinstance(v, str) else v],
             dict(
                 a=dict(
@@ -160,6 +204,7 @@ class TestRamda(TestCase):
             )
         )
 
+    # TODO Failing: [0, and [1 arrays get smashed together and 0 and 1 vanish
     def test_map_keys_deep(self):
         assert {'aCool': {'bCool': [[0, 'c'], [1, {'dCool': 'f'}]]}} == map_keys_deep(
             lambda k, v: f'{k}Cool' if isinstance(k, str) else k,
@@ -174,3 +219,48 @@ class TestRamda(TestCase):
                 )
             )
         )
+
+    def test_fake_lens_path_view(self):
+        result = fake_lens_path_view(key_string_to_lens_path('cherry.strawberry.orange'), dict(
+            cherry=dict(
+                strawberry=dict(
+                    orange='pretzel'
+                )
+            )
+        ))
+        assert result == 'pretzel'
+
+    def test_fake_lens_path_set(self):
+        result = fake_lens_path_set(key_string_to_lens_path('cherry.strawberry.orange'), 'buddy holly', dict(
+            cherry=dict(
+                strawberry=dict(
+                    orange='pretzel'
+                )
+            )
+        ))
+        assert result == dict(
+            cherry=dict(
+                strawberry=dict(
+                    orange='buddy holly'
+                )
+            )
+        )
+
+        class Junior(object):
+            def __init__(self, one):
+                self.cherry = one
+
+        zob = fake_lens_path_set(key_string_to_lens_path('cherry.strawberry.orange'), 'buddy holly', Junior(dict(
+            strawberry=dict(
+                orange='pretzel'
+            )
+        )))
+        assert zob.__dict__ == Junior(dict(
+                strawberry=dict(
+                    orange='buddy holly'
+                )
+        )).__dict__
+
+    def test_props(self):
+        result = props(['myr', 'beite', 'seter'], dict(myr='soggy', beite='tasty', seter='sleepy', avfall='dirty'))
+        assert result == ['soggy', 'tasty', 'sleepy']
